package eu.etaxonomy.taxeditor.prototype2.model;

import java.util.Set;

import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.list.IListChangeListener;
import org.eclipse.core.databinding.observable.list.IObservableList;
import org.eclipse.core.databinding.observable.list.ListChangeEvent;
import org.eclipse.core.databinding.observable.list.ListDiffEntry;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.core.databinding.observable.set.WritableSet;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.view.TaxonomicTreeViewer;

/**
 * Adds tree functionality (get parent, get children) to ObservableListContentProvider;
 *  much code was copied almost in tact from parent class due to
 *  viewer type restrictions and private settings of variables and methods
 * 
 * @author p.ciardelli
 *
 */
public class NameTreeContentProvider extends ObservableListContentProvider 
		implements ITreeContentProvider {

	public Object[] getChildren(Object parentElement) {
		Set<Taxon> children = ((Taxon) parentElement).getTaxonomicChildren();
		observableList.addAll(children);
		return children == null ? new Object[0] : children.toArray();
	}

	public Object getParent(Object element) {
		return ((Taxon) element).getTaxonomicParent();
	}

	public boolean hasChildren(Object element) {
		return ((Taxon) element).hasTaxonomicChildren();
	}
	
	private IObservableList observableList;

	private Viewer viewer;

	private IListChangeListener listener = new IListChangeListener() {
		public void handleListChange(ListChangeEvent event) {
			if (viewer.getControl().isDisposed()) {
				return;
			}
			ListDiffEntry[] differences = event.diff.getDifferences();
			for (int i = 0; i < differences.length; i++) {
				ListDiffEntry entry = differences[i];
				
				if (entry.isAddition()) {
					
					knownElements.add(entry.getElement());
		
					((TaxonomicTreeViewer) viewer).add(
							((Taxon) entry.getElement()).getTaxonomicParent(), 
							entry.getElement());
				} else {

					knownElements.remove(entry.getElement());
					
					((TaxonomicTreeViewer) viewer).remove(entry.getElement());
				}
			}
		}
	};

	private IObservableSet knownElements;

	/**
	 * 
	 */
	public NameTreeContentProvider() {
		
		Assert.isNotNull(Realm.getDefault());
		
		observableList = new WritableList(SWTObservables.getRealm(Display.getDefault()));
		knownElements = new WritableSet(SWTObservables.getRealm(Display.getDefault()));
	}

	public Object[] getElements(Object inputElement) {
		return observableList.toArray();
	}

	public void dispose() {
		observableList.removeListChangeListener(listener);
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		this.viewer = viewer;

		if (newInput != null && !(newInput instanceof IObservableList)) {
			throw new IllegalArgumentException(
					"This content provider only works with input of type IObservableList"); //$NON-NLS-1$
		}

		setInput((IObservableList) newInput);
	}

	/**
	 * @param list
	 */
	private void setInput(IObservableList list) {
		
		if (list == null) {
			list = new WritableList(SWTObservables.getRealm(Display.getDefault()));
		}

		if (observableList != null) {
			observableList.removeListChangeListener(listener);
		}
		
		knownElements.clear();

		observableList = list;
		
		knownElements.addAll(list);
		

		observableList.addListChangeListener(listener);
	}

	/**
	 * @return the set of elements known to this content provider. Label providers may track
	 * this set if they need to be notified about additions before the viewer sees the added
	 * element, and notified about removals after the element was removed from the viewer.
	 */
	public IObservableSet getKnownElements() {
		return knownElements;
	}
}