/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

/**
 * @author p.ciardelli
 * @created 22.05.2008
 * @version 1.0
 */
public class ChangeNameHomotypicGroup extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeNameHomotypicGroup.class);

	private static String text = "Change name's homotypic group";
	private ImageDescriptor image = null;

	private TaxonNameBase name;
	private HomotypicalGroup newHomotypicGroup;

	
	private ChangeNameHomotypicGroup() {
		super(text);
		setImageDescriptor(image);
	}
	
	public ChangeNameHomotypicGroup(TaxonNameBase name, HomotypicalGroup newHomotypicGroup) {
		this();
		
		this.name = name;
		this.newHomotypicGroup = newHomotypicGroup;
	}
	
	public void run() {
		HomotypicalGroup oldHomotypicGroup = name.getHomotypicalGroup();
		if (oldHomotypicGroup != null) {
			oldHomotypicGroup.removeTypifiedName(name);
		}
		
		newHomotypicGroup.addTypifiedName(name);
		
		firePropertyChange(ITaxEditorConstants.HOMOTYPIC_GROUP, oldHomotypicGroup, newHomotypicGroup);		 
	}
}