/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;

/**
 * 1) Changes synonym name's homotypic group, 2) Deletes synonym from its accepted taxon,
 * 3) Creates a new synonym for the accepted taxon
 * @author p.ciardelli
 * @created 22.05.2008
 * @version 1.0
 */
public class ChangeTaxonRelationToSynonymAction extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeTaxonRelationToSynonymAction.class);

	private static String text = "Change synonym's homotypic group";
	private ImageDescriptor image = null;

	private Taxon relatedTaxon;
	private Taxon taxon;
	private HomotypicalGroup newHomotypicGroup;

	private ChangeTaxonRelationToSynonymAction() {
		super(text);
		setImageDescriptor(image);
	}
	
	public ChangeTaxonRelationToSynonymAction(Taxon relatedTaxon, Taxon taxon, HomotypicalGroup newHomotypicGroup) {
		this();
		
		this.relatedTaxon = relatedTaxon;
		this.taxon = taxon;
		this.newHomotypicGroup = newHomotypicGroup;
	}
	
	public void run() {
		
		if (relatedTaxon == null || relatedTaxon.getName() == null || 
				taxon == null || taxon.getName() == null) {
			return;
		}
		
		TaxonNameBase relatedTaxonName = relatedTaxon.getName();
		String synonymType = null;
		Synonym synonym = null;
				
		HomotypicalGroup oldHomotypicGroup = relatedTaxonName.getHomotypicalGroup();
		HomotypicalGroup taxonHomotypicGroup = taxon.getName().getHomotypicalGroup();
		
		new ChangeNameHomotypicGroup(relatedTaxonName, newHomotypicGroup).run();
		
		new DeleteRelatedTaxonFromAcceptedTaxonAction(relatedTaxon, taxon).run();
		if (newHomotypicGroup.equals(taxonHomotypicGroup)) {
			AddHomotypicSynonymToTaxonAction addSynonymAction = new AddHomotypicSynonymToTaxonAction(relatedTaxonName, taxon);
			addSynonymAction.run();
			synonym = addSynonymAction.getSynonym();			
		} else {
			AddHeterotypicSynonymToTaxonAction addSynonymAction = new AddHeterotypicSynonymToTaxonAction(relatedTaxonName, taxon);
			addSynonymAction.run();
			synonym = addSynonymAction.getSynonym();
		}
		
		firePropertyChange(ITaxEditorConstants.HOMOTYPIC_GROUP, null, synonym);		 
	}
}