/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author p.ciardelli
 * @created 09.06.2008
 * @version 1.0
 */
public class SwapSynonymAndTaxonAction extends Action {
	private static final Logger logger = Logger
			.getLogger(SwapSynonymAndTaxonAction.class);
	
	private static String text = "Make synonym accepted name for this taxon";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.SYNONYM_TO_TAXON_ICON);

	private Synonym synonym;
	private Taxon taxon;

	public SwapSynonymAndTaxonAction(Synonym synonym, Taxon taxon) {
		this.synonym = synonym;
		this.taxon = taxon;
	}
	
	public void run() {
		
		// TODO: Call a taxon service method similar to makeTaxonSynonym()
		// instead of implementing the logic here
		
		try {
			IEditorPart oldEditor = EditorController.getEditorByTaxon(taxon);
			if (oldEditor.isDirty()) {
				if (!MessageDialog.openConfirm(GlobalController.getShell(), "Save before proceeding", 
						"All changes must be saved before proceeding with this action.\n\n" +
						"Press \"OK\" to save and continue, or \"Cancel\" to cancel this action.")) {
					return;
				}
				oldEditor.doSave(null);
			}
		} catch (PartInitException e) {
			e.printStackTrace();
			MessageDialog.openError(GlobalController.getShell(), "Error", "Error swapping synonym and taxon");
			return;
		}
		
		taxon.removeSynonym(synonym);
		
		Taxon parentTaxon = taxon.getTaxonomicParent();
		Taxon newTaxon = Taxon.NewInstance(synonym.getName(), taxon.getSec());
		newTaxon.setTaxonomicParent(parentTaxon, null, null);
		
		Synonym newSynonym = Synonym.NewInstance(taxon.getName(), taxon.getSec());
			
		if (CdmUtil.isNameHomotypic(newSynonym.getName(), newTaxon)) {
			newTaxon.addSynonym(newSynonym, SynonymRelationshipType.HOMOTYPIC_SYNONYM_OF());
		} else {
			newTaxon.addSynonym(newSynonym, SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF());			
		}
		
		//	Move the other synonyms from old accepted taxon to new accepted taxon
		for (Synonym synonym: taxon.getSynonyms()) {
			SynonymRelationshipType synRelType = SynonymRelationshipType.HOMOTYPIC_SYNONYM_OF();
			if (CdmUtil.isNameHomotypic(synonym.getName(), newTaxon) != true) {
				synRelType = SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF();
			}
			newTaxon.addSynonym(synonym, synRelType);
			taxon.removeSynonym(synonym);
		}
		
		// Move misapplied names
		for (Taxon misappliedNameTaxon: taxon.getMisappliedNames()) {
			newTaxon.addMisappliedName(misappliedNameTaxon, 
					misappliedNameTaxon.getName().getCitation(), 
					""); //TODO: Set the microcitation
			taxon.removeTaxon(misappliedNameTaxon, TaxonRelationshipType.MISAPPLIED_NAME_FOR());
		}
		
		// Move descriptions
		for(TaxonDescription taxDescription : taxon.getDescriptions()){
			newTaxon.addDescription(taxDescription);
			taxon.removeDescription(taxDescription);
		}
		
		firePropertyChange(ITaxEditorConstants.TAXON, null, newTaxon);
	}	
}