/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.actions.UndoableAction;
import eu.etaxonomy.taxeditor.actions.cdm.ChangeSynonymToTaxonAction;
import eu.etaxonomy.taxeditor.actions.cdm.SaveTaxonAction;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;

/**
 * @author p.ciardelli
 * @created 09.06.2008
 * @version 1.0
 * 
 * @deprecated
 * TODO seems like this is not beeing used at the moment
 */
public class ChangeCompositeToNewTaxonAction extends UndoableAction {
	private static final Logger logger = Logger
			.getLogger(ChangeCompositeToNewTaxonAction.class);

	private static String text = "Use synonym name to make a new taxon";
	private static ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.SYNONYM_TO_TAXON_ICON);

	private NameComposite composite;
	private Taxon taxon;
	private Synonym synonym;
	
	public ChangeCompositeToNewTaxonAction(NameComposite composite,
			Taxon taxon) {
		super(text, image);
		
		this.composite = composite;
		this.taxon = taxon;
		
		if (composite.getData() instanceof Synonym) {
			this.synonym = (Synonym) composite.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires a composite with a Synonym in its data field.");
		}

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.actions.UndoableAction#executeOperation()
	 */
	@Override
	protected IStatus executeOperation() {
		Action action = new ChangeSynonymToTaxonAction(synonym);
		action.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				if (event.getProperty().equals(ITaxEditorConstants.TAXON)) {
					if (event.getNewValue() instanceof Taxon) {
						
						// Delete synonym composite in old taxon
						composite.dispose();
						
						Taxon newTaxon = (Taxon) event.getNewValue(); 
//						TaxEditorPlugin.getDefault().addSessionTaxon(newTaxon);
						
						new OpenTaxonEditorAction(newTaxon).run();
//						new SaveTaxonAction(newTaxon).run();
					}
				}
			}
		});
		action.run();
		return Status.OK_STATUS;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.actions.UndoableAction#redoOperation()
	 */
	@Override
	protected IStatus redoOperation() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.actions.UndoableAction#undoOperation()
	 */
	@Override
	protected IStatus undoOperation() {
		// TODO Auto-generated method stub
		return null;
	}
}
