/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.controller;

import java.util.Set;
import java.util.SortedSet;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.operations.IWorkbenchOperationSupport;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;

/**
 * A collection of useful methods related to the UI.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class GlobalController {
	private static final Logger logger = Logger.getLogger(GlobalController.class);

	private static Set<Feature> preferredFeatureSet;
	private static Set<Rank> preferredRankSet;
	
	/**
	 * @return
	 */
	public static IWorkbenchPage getActivePage() {
		return TaxEditorPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
	}

	/**
	 * @return
	 */
	public static Shell getShell() {
		return TaxEditorPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
	}

	public static IViewPart getViewById(String id) {
		return TaxEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().
				getActivePage().findView(id);
	}
	

	public static IWorkbenchOperationSupport getOperationSupport() {
		return TaxEditorPlugin.getDefault().getWorkbench().
				getOperationSupport();
	}
	
	public static IOperationHistory getOperationHistory() {
//		if (operationHistory == null) {
////			operationHistory = getOperationSupport().getOperationHistory();
//			operationHistory = OperationHistoryFactory.getOperationHistory();
//		}
//		return operationHistory;
		return getOperationSupport().getOperationHistory();
	}

	public static IUndoContext getWorkbenchUndoContext() {
		// TODO make this more specific than GLOBAL_UNDO_CONTEXT
//		return getOperationSupport().getUndoContext();
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}

	public static void setStatusMessage(String msg) {
		PropertySheetController.getPropertySheet().getViewSite().getActionBars().getStatusLineManager().setMessage(msg); 
	}

	/**
	 * Get the name out of the session's name relations vocabulary, not directly
	 * from the object. 
	 * 
	 * @param type
	 * @return
	 */
	public static String getNameRelationLabelType(NameRelationshipType type) {
		SortedSet<NameRelationshipType> vocab = 
				CdmSessionDataRepository.getDefault().getNameRelationshipTypes();
		for (NameRelationshipType type1 : vocab) {
			if (type1.equals(type)) {
				return type1.getLabel();
			}
		}
		return "";
	}
	
	private static IStatusLineManager statusLineManager;

	public static void setStatusLineManager(IStatusLineManager manager) {
		statusLineManager = manager;
	}

	public static void setStatusLine(String message) {
		statusLineManager.setMessage(message);
	}
	
	public static IProgressMonitor getStatusLineProgressMonitor() {
		statusLineManager.setCancelEnabled(false);
		return statusLineManager.getProgressMonitor();
	}

	public static Set<Feature> getPreferredFeatures() {
		return preferredFeatureSet;
	}

	public static void setPreferredFeatures(Set<Feature> preferredFeatureSet) {
		GlobalController.preferredFeatureSet = preferredFeatureSet;
	}

	public static Set<Rank> getPreferredRanks() {
		return preferredRankSet;
	}

	public static void setPreferredRanks(Set<Rank> preferredRankSet) {
		GlobalController.preferredRankSet = preferredRankSet;
	}

	public static void addPreferredRank(Rank rank) {
		preferredRankSet.add(rank);
	}

	public static void addPreferredFeature(Feature feature) {
		GlobalController.preferredFeatureSet.add(feature);		
	}
	
	public static void executeOperation(IUndoableOperation operation){
		try {
			IStatus status = getOperationHistory().execute(operation, getStatusLineProgressMonitor(), null);
			String statusString = status.equals(Status.OK_STATUS) ? "completed" : "cancelled";
			setStatusLine(operation.getLabel() + " " + statusString + ".");
		} catch (ExecutionException e) {
			logger.error("Error executing operation: " + operation.getLabel(), e);
		}
	}

}