/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.controller;

import org.apache.log4j.Logger;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.PropertySheetPage;

import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.propertysheet.CustomSortPropertySheetEntry;

/**
 * @author n.hoffmann
 * @created 20.01.2009
 * @version 1.0
 */
public class PropertySheetController {
	private static final Logger logger = Logger
			.getLogger(PropertySheetController.class);

	public static IViewPart getPropertySheet() {
		for (IViewReference reference : GlobalController.getActivePage().getViewReferences()) {
			if (reference.getId().equals(IPageLayout.ID_PROP_SHEET)) {
				logger.warn(reference.getView(false).getSite().getPart().getTitle());
				return reference.getView(false);
			}
		}
		return null;
	}

	/**
	 * By default, property sheet has buttons in the toolbar for 
	 * "Show advanced properties" and "Show categories".
	 * <p>
	 * This is confusing for the user, hence a method to remove them
	 * until such time as advanced properties or categories are implemented.
	 */
	public static void hidePropertySheetToolbar() {
		PropertySheet propertySheet = (PropertySheet) getPropertySheet();
		IActionBars actionBars = propertySheet.getViewSite().getActionBars();
		actionBars.getToolBarManager().removeAll();
		actionBars.getMenuManager().removeAll();
	}

	/**
		 * The property sheet listener ensures only property sheets
		 * 	with data cause the Property Sheet to be updated.
		 */
		public static void addPropertySheetInputListener() {
			IViewPart propertySheet = getPropertySheet();
	//		propertySheet.get
			PropertySheet ps = (PropertySheet) propertySheet;
	//		ps.addPartPropertyListener(listener)
	//		ps.addPropertyListener(l)
		}

	public static void setPropertySheetTree(Tree tree) {
		TaxEditorPlugin.getDefault().setPropertySheetTree(tree);		
	}

	public static Tree getPropertySheetTree() {
		return TaxEditorPlugin.getDefault().getPropertySheetTree();		
	}

	public static void setPropertySheetPage(PropertySheetPage page) {
		TaxEditorPlugin.getDefault().setPropertySheetPage(page);
	}

	public static PropertySheetPage getPropertySheetPage() {
		return TaxEditorPlugin.getDefault().getPropertySheetPage();	
	}

	/**
	 * 		UiUtil.paintPropertySheetRow(P_DATEPUBLISHED, new Color(Display.getDefault(), WarningAnnotation.WARNING_RGB), true);
	 * 		UiUtil.unpaintPropertySheetRow(P_DATEPUBLISHED);
	 * 
	 * @param id
	 * @param color
	 * @param doPaintChildren
	 */
	public static void paintPropertySheetRow(String id, Color color, boolean doPaintChildren) {
		
		// Catch null property sheet name
		if (id == null) {
			return;
		}
		
		// Catch uninit'ed property sheet tree
		if (getPropertySheetTree() == null) {
			return;
		}
		
		paintPropertySheetRow(id, color, doPaintChildren, getPropertySheetTree());
	}

	private static void paintPropertySheetRow(String id, Color color, boolean doPaintChildren, Object treeOrItem) {
		
		// Init items w zero-length array
		TreeItem[] items = new TreeItem[]{};
		
		// Get child items depending to class
		if (treeOrItem instanceof Tree) {
			items = ((Tree) treeOrItem).getItems();
		}
		if (treeOrItem instanceof TreeItem) {
			items = ((TreeItem) treeOrItem).getItems();
		}
		
		// If array hasn't been populated by the above, return
		if (items.length == 0) {
			return;
		}
	
		// Prop. sheet id's take the form "01:xxxx" for sorting - truncate
		id = CustomSortPropertySheetEntry.truncateDisplayName(id);
		
		// Iterate through child items
		for (TreeItem item : items) {
	
			// Item found, paint it
			if (id.equals(item.getText())) {
				paintItem(item, color, doPaintChildren);
				return;
			}			
	
			// Recursively search for item to paint in child items
			if (item.getItemCount() > 0) {
				paintPropertySheetRow(id, color, doPaintChildren, item);
			}
		}	
	}

	public static void unpaintPropertySheetRow(String id) {
		
		// Catch uninit'ed property sheet tree
		if (getPropertySheetTree() == null) {
			return;
		}
		
		// Get tree's background color to "unpaint"
		Color color = getPropertySheetTree().getBackground();
		
		paintPropertySheetRow(id, color, true);
	}

	/**
	 * Note: children are only painted if submenu has already been created, i.e. opened once.
	 * 
	 * @param item
	 * @param color
	 * @param doPaintChildren
	 */
	public static void paintItem(TreeItem item, Color color, boolean doPaintChildren) {
		
		// Paint the item
		item.setBackground(color);
		
		// Recursively paint child items if requested
		if (doPaintChildren) {
			for (TreeItem childItem : item.getItems()) {
				paintItem(childItem, color, doPaintChildren);
			}
		}
	}
	
}
