/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.transaction.TransactionStatus;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author p.ciardelli
 * @created 16.12.2008
 * @version 1.0
 */
public class CdmTransactionController {
	private static final Logger logger = Logger
			.getLogger(CdmTransactionController.class);
	
	private static CdmApplicationController applicationController;
	private static TransactionStatus txStatus;
	private CdmTransactionController controller;
		
	public static void startTransaction() {
		
		txStatus = getCdmAppController().startTransaction();
		
		// Workaround - nom statii were causing a lazy loading error
		List<NomenclaturalStatusType> list = new ArrayList<NomenclaturalStatusType>();
		list.add(NomenclaturalStatusType.SUPERFLUOUS());
		list.add(NomenclaturalStatusType.NUDUM());
		list.add(NomenclaturalStatusType.ILLEGITIMATE());
		list.add(NomenclaturalStatusType.INVALID());
		list.add(NomenclaturalStatusType.CONSERVED());
		list.add(NomenclaturalStatusType.ALTERNATIVE());
		list.add(NomenclaturalStatusType.REJECTED());
		list.add(NomenclaturalStatusType.REJECTED_PROP());
		list.add(NomenclaturalStatusType.PROVISIONAL());
		list.add(NomenclaturalStatusType.SUBNUDUM());
		list.add(NomenclaturalStatusType.OPUS_UTIQUE_OPPR());
		list.add(NomenclaturalStatusType.AMBIGUOUS());
		list.add(NomenclaturalStatusType.DOUBTFUL());
		list.add(NomenclaturalStatusType.CONFUSUM());
		list.add(NomenclaturalStatusType.UTIQUE_REJECTED());
		list.add(NomenclaturalStatusType.CONSERVED_PROP());
		list.add(NomenclaturalStatusType.LEGITIMATE());
		list.add(NomenclaturalStatusType.NOVUM());
		list.add(NomenclaturalStatusType.ORTHOGRAPHY_CONSERVED());
		list.add(NomenclaturalStatusType.SANCTIONED());
		list.add(NomenclaturalStatusType.COMBINATION_INVALID());
		list.add(NomenclaturalStatusType.VALID());
		for (NomenclaturalStatusType status : list){
			applicationController.getTermService().saveTerm(status);
		}
	}

	public static void commitTransaction() {
		
		// Make sure transaction exists
		if (txStatus == null) {
			throw new IllegalStateException("Transaction was NULL - be sure to call startTransaction first.");
		}
		
		getCdmAppController().commitTransaction(txStatus);		
		txStatus = null;
	}
	
	public static void setApplicationController(CdmApplicationController applicationController) {
		CdmTransactionController.applicationController = applicationController;
	}
	
	private static CdmApplicationController getCdmAppController() {
		
		// Make sure the app controller has been set
		if (applicationController == null) {
			throw new IllegalStateException("CdmApplicationController not yet set.");
		}
		return applicationController;
	}

	public static void addTaxonToTransaction(Taxon taxon) {
		CdmUtil.getTaxonService().saveTaxon(taxon);
	}
	
	public static void addSessionTaxaToTransaction() {
		CdmUtil.getTaxonService().saveTaxonAll(
				CdmSessionDataRepository.getDefault().getAllTaxa());
	}
}
