/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.actions.WidgetTransfer;

/**
 * @author p.ciardelli
 * @created 25.06.2008
 * @version 1.0
 */
public class GroupComposite extends Composite {
	private static final Logger logger = Logger.getLogger(GroupComposite.class);
	
	private DropTarget target;
	private DropTargetListener dropListener;
		
	public GroupComposite(Composite parent) {
		super(parent, SWT.NONE);

		// Graphic layout of composite
		this.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		this.setLayout(new TableWrapLayout());
		Color groupBackgroundColor = TaxEditorPlugin.getDefault().
				getColor(ITaxEditorConstants.GROUP_GRAY_BKG_COLOR);
		this.setBackground(groupBackgroundColor);
//		this.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));

		// Drop functionality
		this.setDroppable(true);

	}
	
	public void setDroppable(boolean droppable) {
		if (droppable) {
			Transfer[] types = new Transfer[] {WidgetTransfer.getInstance()};	
			int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
			target = new DropTarget(this, operations);
			target.setTransfer(types);
			dropListener = new DropTargetAdapter() {
							
				public void dragEnter(DropTargetEvent event) {
					GroupComposite.this.notifyListeners(SWT.FocusIn, null);
				}
				
				public void dragLeave(DropTargetEvent event) {
					GroupComposite.this.notifyListeners(SWT.FocusOut, null);
				}
				
				public void drop(DropTargetEvent event) {
										
					if (event.data instanceof Composite) {
						Composite composite = (Composite) event.data; 
						if ( ! composite.getParent().equals(GroupComposite.this)) {
							composite.setParent(GroupComposite.this);
						}
					}
				}
			};
			target.addDropListener(dropListener);			
		} else {
			if (dropListener != null) {
				target.removeDropListener(dropListener);
			}
		}
	}
	
	public void dispose() {
		
		target.removeDropListener(dropListener);
		
		for (Control control : getChildren()) {
			if (!control.isDisposed()) {
				control.dispose();
			}
		}
		
		super.dispose();
	}
}
