/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.taxeditor.actions.WidgetTransfer;

/**
 * A composite which can be dragged between different EditorGroupComposite. The user
 * uses the draggable control for dragging the whole composite.
 * 
 * @see eu.etaxonomy.taxeditor.editor.EditorGroupComposite
 * 
 * @author p.ciardelli
 * 
 */
abstract public class GroupedComposite extends Composite {
	private static final Logger logger = Logger.getLogger(GroupedComposite.class);

	private Control draggableControl;

	private DragSource dragSource;
	
	public GroupedComposite(Composite parent) {
		super(parent, SWT.NONE);

		createContent();
	}

	abstract protected void createContent();

	protected void setDraggableControl(Control control) {
		draggableControl = control;
	}

	public void setDraggable(boolean draggable) {

		if (draggable) {

			if (dragSource != null) {
				// Already initialized
				return;
			}

			if (draggableControl == null) {
				throw new NullPointerException(
						"Draggable control must be set to add draggability");
			}

			WidgetTransfer widgetTransfer = WidgetTransfer.getInstance();
			widgetTransfer.setWidget(this);
			Transfer[] types = new Transfer[] { widgetTransfer };
			int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;

			dragSource = new DragSource(draggableControl, operations);
			dragSource.setTransfer(types);
			dragSource.addDragListener(dragSourceListener);

		} else {
			dragSource = null;
		}
	}

	/**
	 * Drag listener which passes the Composite as the data in a drag event.
	 */
	DragSourceListener dragSourceListener = new DragSourceAdapter() {

		public void dragStart(DragSourceEvent event) {
			GroupedComposite.this.setFocus();
			event.doit = true;
		}
		
		public void dragFinished(DragSourceEvent event){
//			((DragSource) event.widget).dispose();
		}
	};
	
	public void dispose () {
		dragSource.removeDragListener(dragSourceListener);
		super.dispose();
	}
}