/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.taxon.Taxon;

/**
 * @author p.ciardelli
 * @created 27.06.2008
 * @version 1.0
 */
public class TaxonomicTreeContentProvider implements ITreeContentProvider {
	private static final Logger logger = Logger
			.getLogger(TaxonomicTreeContentProvider.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) {
		
		String errorMessage = "getChildren(parentElement) called with non-Taxon element.";
		
		// Make sure we are dealing with taxon objects
		Assert.isTrue(
				parentElement instanceof Taxon || parentElement instanceof Collection,
				errorMessage);
		
		// Return children for a single taxon
		if (parentElement instanceof Taxon) {
			return getTaxonChildren((Taxon) parentElement).toArray();
		}

		// Return children for a collection of taxa
		List<Taxon> children = new ArrayList<Taxon>();
		for (Object parent : (Collection) parentElement) {

			Assert.isTrue(parent instanceof Taxon, errorMessage);
			children.addAll(getTaxonChildren((Taxon) parent));
		}
		return children.toArray();
	}

	private Set<Taxon> getTaxonChildren(Taxon parentTaxon) {
		return CdmSessionDataRepository.getDefault().getTaxonomicChildren(parentTaxon);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		
		Assert.isTrue(element instanceof Taxon, 
				"getParent(element) called with non-Taxon element.");
		
		return ((Taxon) element).getTaxonomicParent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		Assert.isTrue(element instanceof Taxon, 
				"hasChildren(element) called with non-Taxon element.");
		
		return ((Taxon) element).getTaxonomicChildrenCount() > 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		
		// BIG QUESTION: should getElements return only root taxa, or all taxa?
		// CONFLICTING INFO ONLINE!!!
//		return TaxEditorPlugin.getDefault().getObservableSessionTaxa().toArray();
		
		// Is this merely a method to convert elements into an array?
		// OR should children be returned?
		
		return CdmSessionDataRepository.getDefault().getRootTaxa().toArray();
//		return getChildren(inputElement);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
	 *      java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub
	}
}