/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.controller.EditorController;

/**
 * @author p.ciardelli
 * @created 19.01.2009
 * @version 1.0
 */
public class SearchResultView extends ViewPart {
	private static final Logger logger = Logger
			.getLogger(SearchResultView.class);

	public static final String ID = "eu.etaxonomy.taxeditor.navigation.searchresultview"; //$NON-NLS-1$

	private TableViewer resultViewer;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		resultViewer = new TableViewer(parent, SWT.BORDER);
		resultViewer.setContentProvider(new ArrayContentProvider());
		resultViewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				if (element instanceof TaxonBase) {
					return ((TaxonBase) element).getTitleCache();
				}
				return super.getText(element);
			}
		});
		resultViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				TaxonBase taxonBase = null;
				try {
					taxonBase = (TaxonBase) ((StructuredSelection) event.getSelection())
							.getFirstElement();
					
					openTaxonEditor(taxonBase);
					
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	protected void openTaxonEditor(TaxonBase taxonBase) {

		Taxon taxon = null;
		if (taxonBase instanceof Synonym) {
			taxon = (Taxon) ((Synonym) taxonBase).getAcceptedTaxa().toArray()[0];
		} else {
			taxon = (Taxon) taxonBase;
		}

		// Open a taxon editor
		EditorController.open(taxon);

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		// TODO Auto-generated method stub

	}
	private void initializeToolBar() {
		IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
	}

	public void setResult(SearchResult searchResult) {
		resultViewer.setInput(searchResult); 
		setPartName("Search: '" + searchResult.getSearchString() + "'");
	}
}
