/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.navigation;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.part.ViewPart;

import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.actions.ui.OpenTaxonEditorAction;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceDialog;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;

/**
 * The panel containing the taxonomic tree.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class TaxonomicTreeView extends ViewPart {
	private static final Logger logger = Logger.getLogger(TaxonomicTreeView.class);
	
	private Composite dbComposite;
	private Composite taxTreeComposite = null;
	private Composite top = null;
	public static final String ID = "eu.etaxonomy.taxeditor.navigation.taxonomictreeview"; //$NON-NLS-1$
	
	private TaxonomicTreeViewer treeViewer;
	private IManagedForm hyperlinkForm;
	private Menu dataSourceMenu;
	
	/**
	 * Create contents of the view part
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
				
		// Layout top composite
		parent.setLayout(new FillLayout());
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		gridLayout.verticalSpacing = 0;	
				
		// Create top composite
		top = new Composite(parent, SWT.NONE);
		top.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		top.setLayout(gridLayout);

		// Create composite for DB
		dbComposite = new Composite(top, SWT.NONE);
		dbComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		dbComposite.setLayout(gridLayout_1);
		final Label lblDataSourceTitle = new Label(dbComposite, SWT.NONE);
		lblDataSourceTitle.setFont(SWTResourceManager.getFont("Arial", 14, SWT.BOLD));
		lblDataSourceTitle.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		lblDataSourceTitle.setAlignment(SWT.CENTER);
		lblDataSourceTitle.setText(" Local");		
		
		// Create DB label
		final Label lblDatasourceMenu = new Label(dbComposite, SWT.NONE);
		lblDatasourceMenu.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		lblDatasourceMenu.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		lblDatasourceMenu.setImage(ResourceManager.getPluginImage(TaxEditorPlugin.getDefault(), "icons/db.gif"));
		lblDatasourceMenu.setImage(TaxEditorPlugin.getDefault().getImageRegistry().get(ITaxEditorConstants.DB_ICON));

		// Create menu to hang from DB label
		dataSourceMenu = new Menu(lblDatasourceMenu);
		lblDatasourceMenu.setMenu(dataSourceMenu);
		createDataSourceMenu();

		// Show current DB
		final Label lblDataSource = new Label(dbComposite, SWT.NONE);
		final GridData gd_lblDataSource = new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1);
		gd_lblDataSource.heightHint = 16;
		lblDataSource.setLayoutData(gd_lblDataSource);
		lblDataSource.setFont(SWTResourceManager.getFont("Arial", 8, SWT.NONE));
		lblDataSource.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		lblDataSource.setAlignment(SWT.CENTER);
		lblDataSource.setText("  Datasource: local h2 database");

		// Create composite for taxonomic tree
		taxTreeComposite = new Composite(top, SWT.NONE);
		taxTreeComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		taxTreeComposite.setLayout(new FillLayout());
		taxTreeComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		
		// If there are no root taxa, provide a link to create this tree's initial root taxon
		if (CdmSessionDataRepository.getDefault().getRootTaxa().size() == 0) {
			
			hyperlinkForm = new ManagedForm(taxTreeComposite);
			Composite linkBody = hyperlinkForm.getForm().getBody();
			
			TableWrapLayout linkLayout = new TableWrapLayout();
			linkLayout.leftMargin = 20;
			linkLayout.rightMargin = 20;
			linkBody.setLayout(linkLayout);
			linkBody.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			
			Hyperlink link = hyperlinkForm.getToolkit().createHyperlink(linkBody, 
					"Your data source is empty. Click here to create a root taxon.", SWT.WRAP);
			link.addHyperlinkListener(new HyperlinkAdapter() {

				@Override
				public void linkActivated(HyperlinkEvent e) {
					new OpenTaxonEditorAction().run();
				}				
			});
			
			treeViewer = null;
			
		} else {
			createTreeViewer();
		}

		createActions();
		initializeToolBar();
		initializeMenu();
	}
	
	private void createDataSourceMenu() {

		// Create entry to open data source dialog
		new MenuItem(dataSourceMenu, SWT.SEPARATOR);
		final MenuItem newDataSourceMenuItem = new MenuItem(dataSourceMenu, SWT.NONE);
		newDataSourceMenuItem.setText("Edit data sources ...");
		newDataSourceMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				Dialog dialog = new CdmDataSourceDialog(GlobalController.getShell());
				dialog.open();
				
				// Refresh menu to reflect any changes in data 
				refreshDataSources();
			}
		});
		
		// Add data sources to menu
		refreshDataSources();
	}

	private void refreshDataSources() {
		
		// First dispose of any entries that are already there
		for (MenuItem menuItem : dataSourceMenu.getItems()) {
			if (menuItem.getData() instanceof ICdmDataSource) {
				menuItem.dispose();
			}
		}
				
		// Get current datasource and list of all data sources 
		ICdmDataSource sessionDataSource = CdmDataSourceRepository.getDefault().
				getCurrentDataSource();
		List<ICdmDataSource> dataSources = 
				CdmDataSourceRepository.getDefault().getAll();
		
		// Create menu items, putting a check next to the current one
		for (ICdmDataSource dataSource : dataSources) {
			
			// Always add menu item to index 0, i.e. at the beginning of the menu
			MenuItem item = new MenuItem(dataSourceMenu, SWT.CHECK, 0);
			
			// If this is the current data source, put a check by it
			if (dataSource.equals(sessionDataSource)) {
				item.setSelection(true);
			}
			
			// Make a nice little name string out of this mutha
			String text = dataSource.getName();
			if (!dataSource.getDatabaseType().equals(DatabaseTypeEnum.H2)) {
				if (dataSource.getDatabase() != null) {
					text += " (" + dataSource.getDatabaseType().getName() + ")";
				}
			}
			item.setText(text);
			item.setData(dataSource);
			
			// On selection, make this the current data source
			item.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					
					// Make sure widget has a datasource in its data field
					if (!(e.widget.getData() instanceof ICdmDataSource)) {
						return;
					}
					
					// Get the datasource from the widget's data field
					ICdmDataSource dataSource = (ICdmDataSource) e.widget.getData(); 
						
					// Make sure this isn't already selected
					if (dataSource.equals(CdmDataSourceRepository.getDefault().getCurrentDataSource())) {
						
						// Make sure it remains checked, then return
						if (e.widget instanceof MenuItem) {
							((MenuItem) e.widget).setSelection(true);
						}
						return;
					}
					
					// Set datasource in widget's data to current datasource
					CdmDataSourceRepository.getDefault().setCurrentDataSource(dataSource);
					
					// Deselect all other datasources
					for (MenuItem item : dataSourceMenu.getItems()) {
						if (item.getData() != null && !item.getData().equals(dataSource)) {
							item.setSelection(false);
						}
					}
				}
			});
		}
	}
	
	public TaxonomicTreeViewer createTreeViewer() {
		
		// Dispose link to new taxon as necessary
		if (hyperlinkForm != null) {
			hyperlinkForm.getForm().dispose();
		}
				
		// Create tree
		treeViewer = new TaxonomicTreeViewer(taxTreeComposite);
		
		((TaxonomicTreeViewer) treeViewer).createDoubleClickListener();
		((TaxonomicTreeViewer) treeViewer).createMenu();
		((TaxonomicTreeViewer) treeViewer).createQuickAdd();
		((TaxonomicTreeViewer) treeViewer).createDragAndDrop();
				
		taxTreeComposite.layout();
		
		return treeViewer;
	}
	
	private void createActions() {
		// Create the actions
	}

	/**
	 * Initialize the toolbar
	 */
	private void initializeToolBar() {
		IToolBarManager toolbarManager = getViewSite().getActionBars()
				.getToolBarManager();
	}

	/**
	 * Initialize the menu
	 */
	private void initializeMenu() {
		IMenuManager menuManager = getViewSite().getActionBars()
				.getMenuManager();
	}

	public void setFocus() {
		// Set the focus
	}
	
	/**
	 * Expose TreeViewer, i.e. to manipulate tree nodes
	 * @return
	 */
	public TaxonomicTreeViewer getTreeViewer() {
		return treeViewer;
	}
}
